#include <geos/algorithm/BoundaryNodeRule.h>
#include <geos/geom/CoordinateArraySequence.h>
#include <geos/geom/Geometry.h>
#include <geos/geom/GeometryFactory.h>
#include <geos/geom/IntersectionMatrix.h>
#include <geos/geom/LineString.h>
#include <geos/geom/PrecisionModel.h>
#include <geos/operation/relate/RelateOp.h>

#include <cstdlib> // exit()

using namespace geos;
using namespace geos::geom;

using geos::algorithm::BoundaryNodeRule;
using geos::geom::IntersectionMatrix;
using geos::operation::relate::RelateOp;

GeometryFactory::Ptr global_factory;

LineString* create_closed_triangle_linestring(double x1, double y1, double x2, double y2, double x3, double y3)
{
  CoordinateArraySequence* cl = new CoordinateArraySequence();

  cl->add(Coordinate(x1, y1));
  cl->add(Coordinate(x2, y2));
  cl->add(Coordinate(x3, y3));
  cl->add(Coordinate(x1, y1));

  LineString* ls = global_factory->createLineString(cl);
  return ls;
}


int main()
{
  PrecisionModel* pm = new PrecisionModel(2.0, 0, 0);
  global_factory = GeometryFactory::create(pm, -1);
  delete pm;

  geos::geom::LineString* triangle1 =
    create_closed_triangle_linestring(0, 0, 1, 2, 2, 0);
  geos::geom::LineString* triangle2 =
    create_closed_triangle_linestring(1.5, 1.5, 2.5, 3.5, 3.5, 1.5);
  geos::geom::LineString* triangle3 =
    create_closed_triangle_linestring(3, 3, 4, 5, 5, 3);


  //CORRECT RESULT
  std::unique_ptr<IntersectionMatrix> imMod2 = RelateOp::relate(
    triangle1, triangle2, BoundaryNodeRule::getBoundaryRuleMod2());
  std::cout << "Bounding boxes intersect, mod-2 rule: " << imMod2->toString() << "\n";

  //CORRECT RESULT
  std::unique_ptr<IntersectionMatrix> imEndPoint = RelateOp::relate(
    triangle1, triangle2, BoundaryNodeRule::getBoundaryEndPoint());
  std::cout << "Bounding boxes intersect, endpoint rule: " << imEndPoint->toString() << "\n";

  //CORRECT RESULT
  std::unique_ptr<IntersectionMatrix> im2Mod2 = RelateOp::relate(
    triangle1, triangle3, BoundaryNodeRule::getBoundaryRuleMod2());
  std::cout << "Bounding boxes do not intersect, mod-2 rule: " << im2Mod2->toString() << "\n";

  //INCORRECT RESULT
  std::unique_ptr<IntersectionMatrix> im2EndPoint = RelateOp::relate(
    triangle1, triangle3, BoundaryNodeRule::getBoundaryEndPoint());
  std::cout << "Bounding boxes do not intersect, endpoint rule: " << im2EndPoint->toString() << "\n";

  exit(0);
}

